@@ -1,32 +1,37 @@
 # "Load" the data. In this case we are using data built into the package.
 # In practice, users will likely load raster data using the raster() function
 # from the raster package.
-res_data <- samc::ex_res_data
-abs_data <- samc::ex_abs_data
-occ_data <- samc::ex_occ_data
+res_data <- samc::example_split_corridor$res
+abs_data <- samc::example_split_corridor$abs
+init_data <- samc::example_split_corridor$init
 
 
 # Make sure our data meets the basic input requirements of the package using
 # the check() function.
 check(res_data, abs_data)
-check(res_data, occ_data)
+check(res_data, init_data)
+
+# Setup the details for a random-walk model
+rw_model <- list(fun = function(x) 1/mean(x), # Function for calculating transition probabilities
+                 dir = 8, # Directions of the transitions. Either 4 or 8.
+                 sym = TRUE) # Is the function symmetric?
 
 
 # Create a `samc-class` object with the resistance and absorption data using
 # the samc() function. We use the recipricol of the arithmetic mean for
 # calculating the transition matrix. Note, the input data here are matrices,
-# not RasterLayers. If using RasterLayers, the latlon parameter must be set.
-samc_obj <- samc(res_data, abs_data, tr_fun = function(x) 1/mean(x))
+# not RasterLayers.
+samc_obj <- samc(res_data, abs_data, model = rw_model)
 
 
-# Convert the occupancy data to probability of occurrence
-occ_prob_data <- occ_data / sum(occ_data, na.rm = TRUE)
+# Convert the initial state data to probabilities
+init_prob_data <- init_data / sum(init_data, na.rm = TRUE)
 
 
 # Calculate short- and long-term metrics using the analytical functions
-short_mort <- mortality(samc_obj, occ_prob_data, time = 50)
+short_mort <- mortality(samc_obj, init_prob_data, time = 50)
 short_dist <- distribution(samc_obj, origin = 3, time = 50)
-long_disp <- dispersal(samc_obj, occ_prob_data)
+long_disp <- dispersal(samc_obj, init_prob_data)
 visit <- visitation(samc_obj, dest = 4)
 surv <- survival(samc_obj)
 
